// Modul...: UART0.C
// Version.: 2.0
// Compiler: KEIL-ARM
// Datum...: Juni 2005
// Autor...: Udo Jrss
//-----------------------------------------------------------------------------

#include "uart0.h"
//-----------------------------------------------------------------------------

U08 uart0_rx_buffer[UART0_RX_BUFFER_SIZE]; // Empfangspuffer
U08 uart0_tx_buffer[UART0_TX_BUFFER_SIZE]; // Sendepuffer
volatile BOOL uart0_rx_ready; // Globales Flag fr Empfangsbereitschaft

static U32 uart0_rx_buffer_index; // Index von Empfangspuffer
static U08 uart0_rx_char; // Empfangsvariable fr ISR
//-----------------------------------------------------------------------------

void uart0_isr(void) __irq // UART0 Interrupt-Service-Routine
{
  switch ((U0IIR >> 1) & 0x07)
  {
    case IIR_THRE: // Transmitter-Hold-Register ist leer
      U0THR = uart0_rx_char;
      break;
    case IIR_RSL: // Line-Status-Fehler
      uart0_rx_char = U0LSR;
      break;
    case IIR_RDA: // Byte empfangen
    case IIR_CTI: // Timeout
      uart0_rx_char = U0RBR;

      if (uart0_rx_char == CHAR_LF)
      {
        break;
      }

      if (uart0_rx_char == CHAR_CR)
      {
        uart0_rx_buffer[uart0_rx_buffer_index] = 0;
        uart0_rx_buffer_index = 0;
        uart0_rx_ready = TRUE;
        break;
      }     
      
      if (uart0_rx_char == CHAR_BS)
      {
        if (uart0_rx_buffer_index > 0)
        {
          uart0_rx_buffer[uart0_rx_buffer_index--] = 0;
        }
        break;
      }

      if (uart0_rx_buffer_index < UART0_RX_BUFFER_SIZE - 1)
      {
        if (uart0_rx_char >= 'a' && uart0_rx_char <= 'z') uart0_rx_char -= 32;
        uart0_rx_buffer[uart0_rx_buffer_index++] = uart0_rx_char;
      }
      break;

    default:
      break;
  }

  VICVectAddr = 0; // Interrupt in VIC lschen
}
//-----------------------------------------------------------------------------

void uart0_init(U32 baudrate)
{
  PCONP |= PCURT0; // Power-Control fr UART0 einschalten
  PINSEL0 |= UART0_IOSET_MASK; // RxD0 und TxD0 Funktionen einschalten
  U0LCR |= UART_DLAB; // Baudrate setzen
  baudrate = sys_get_fpclk() / baudrate;
  U0DLL = baudrate & 0x00FF;
  U0DLM = (baudrate >> 8) & 0x00FF;
  U0LCR = UART_WLS1 | UART_WLS0; // 8 Bit Wortlnge, 1 Stopbit, keine Paritt
  sys_set_vectored_irq(uart0_isr,UART0_SLOT,VIC_UART0);

  //U0IER = UART_RDAIE; // Receiver Interrupt freischalten
}
//-----------------------------------------------------------------------------

void uart0_write_char(U08 byte)
{
  while (!(U0LSR & UART_THRE)) ; // Warten bis Senderegister leer ist

  U0THR = byte;
}
//-----------------------------------------------------------------------------

void uart0_write_crlf(void)
{
  uart0_write_char(CHAR_CR);
  uart0_write_char(CHAR_LF);
}
//-----------------------------------------------------------------------------

void uart0_write_str(pU08 pstr)
{
  while (*pstr)
  {
    uart0_write_char(*pstr++);
  }
}
//-----------------------------------------------------------------------------

void uart0_writeln_str(pU08 pstr)
{
  uart0_write_str(pstr);
  uart0_write_crlf();
}
//-----------------------------------------------------------------------------

void uart0_write_U08(U08 data,U08 digits) // Byte ausgeben
{
  uart0_write_str(sup_U08_to_str(data,digits));
}
//------------------------------------------------------------------------------

void uart0_writeln_U08(U08 data,U08 digits) // Byte ausgeben
{
  uart0_write_str(sup_U08_to_str(data,digits));
  uart0_write_crlf();
}
//------------------------------------------------------------------------------

void uart0_write_S08(S08 data,U08 digits) // Byte ausgeben
{
  uart0_write_str(sup_S08_to_str(data,digits));
}
//------------------------------------------------------------------------------

void uart0_writeln_S08(S08 data,U08 digits) // Byte ausgeben
{
  uart0_write_str(sup_S08_to_str(data,digits));
  uart0_write_crlf();
}
//------------------------------------------------------------------------------

void uart0_write_U08_hex(U08 data) // Byte hexadezimal ausgeben
{
  uart0_write_str(sup_U08_to_hex(data));
}
//------------------------------------------------------------------------------

void uart0_writeln_U08_hex(U08 data) // Byte hexadezimal ausgeben
{
  uart0_write_str(sup_U08_to_hex(data));
  uart0_write_crlf();
}
//------------------------------------------------------------------------------

void uart0_write_U16(U16 data,U08 digits) // Word dezimal ausgeben
{
  uart0_write_str(sup_U16_to_str(data,digits));
}
//------------------------------------------------------------------------------

void uart0_writeln_U16(U16 data,U08 digits) // Word dezimal ausgeben
{
  uart0_write_str(sup_U16_to_str(data,digits));
  uart0_write_crlf();
}
//------------------------------------------------------------------------------

void uart0_write_U16_hex(U16 data) // Word hexadezimal ausgeben
{
  uart0_write_str(sup_U16_to_hex(data));
}
//------------------------------------------------------------------------------

void uart0_writeln_U16_hex(U16 data) // Word hexadezimal ausgeben
{
  uart0_write_str(sup_U16_to_hex(data));
  uart0_write_crlf();
}
//------------------------------------------------------------------------------

void uart0_write_S16(S16 value,U08 digits)
{
  uart0_write_str(sup_S16_to_str(value,digits));
}
//-----------------------------------------------------------------------------

void uart0_writeln_S16(S16 value,U08 digits)
{
  uart0_write_str(sup_S16_to_str(value,digits));
  uart0_write_crlf();
}
//-----------------------------------------------------------------------------

void uart0_write_U32(U32 value,U08 digits)
{
  uart0_write_str(sup_U32_to_str(value,digits));
}
//-----------------------------------------------------------------------------

void uart0_writeln_U32(U32 value,U08 digits)
{
  uart0_write_str(sup_U32_to_str(value,digits));
  uart0_write_crlf();
}
//-----------------------------------------------------------------------------

void uart0_write_S32(S32 value,U08 digits)
{
  uart0_write_str(sup_S32_to_str(value,digits));
}
//-----------------------------------------------------------------------------

void uart0_writeln_S32(S32 value,U08 digits)
{
  uart0_write_str(sup_S32_to_str(value,digits));
  uart0_write_crlf();
}
//-----------------------------------------------------------------------------

void uart0_write_U32_hex(U32 value)
{
  uart0_write_str(sup_U32_to_hex(value));
}
//-----------------------------------------------------------------------------

void uart0_writeln_U32_hex(U32 value)
{
  uart0_write_str(sup_U32_to_hex(value));
  uart0_write_crlf();
}
//-----------------------------------------------------------------------------

void uart0_write_float(float value)
{
  sprintf((char *)uart0_tx_buffer,"%f",value);
  uart0_write_str(uart0_tx_buffer);
}
//-----------------------------------------------------------------------------

void uart0_writeln_float(float value)
{
  sprintf((char *)uart0_tx_buffer,"%f\r\n",value);
  uart0_write_str(uart0_tx_buffer);
}
